/*=============================================================================
daxtraq - User Management UI
dx_mm.c - Module management
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libHX.h>
#include "dxt/global.h"
#include "dxt/mm.h"
#ifdef _WIN32
#  define dlopen(filename, flags) LoadLibrary(filename)
#  define dlsym(handle, symbol) GetProcAddress(handle, symbol)
#  define dlclose(handle) FreeLibrary(handle)
#endif

static void mm_handle_file(char *, struct ds_module **);

struct ds_module *mm_input = NULL;

//-----------------------------------------------------------------------------
void mm_register_ds(void) {
  // Scan directories for .so and .dll files and load them
  const char *dirs[] = {".", "/usr/lib/daxtraq", NULL};
  struct ds_module *imtrav = (struct ds_module *)&mm_input;
  const char **dirp = dirs;

  while(*dirp != NULL) {
    struct HXdir *dtravp = HXdir_open((char *)*dirp);
    char *fn;

    while((fn = HXdir_read(dtravp)) != NULL) {
      char buf[MAXLNLEN];
      snprintf(buf, MAXLNLEN, "%s/%s", *dirp, fn);
      mm_handle_file(buf, &imtrav);
    }

    HXdir_close(dtravp);
    ++dirp;
  }
  return;
}

void mm_unregister_ds(void) {
  struct ds_module *imtrav = mm_input, *nx;
  while(imtrav != NULL) {
    dlclose(imtrav->handle);
    nx = imtrav->Next;
    free(imtrav);
    imtrav = nx;
  }
  return;
}

//-----------------------------------------------------------------------------
static void mm_handle_file(char *fn, struct ds_module **imtrav) {
  struct ds_module *i = malloc(sizeof(struct ds_module));
  char *extp = strrchr(fn, '.');
  char *type;

  if(extp == NULL) { return; }
  if(strcmp(extp, ".so") != 0 && strcmp(extp, ".dll") != 0) { return; }

  if((i->handle = dlopen(fn, RTLD_NOW)) == NULL) {
    fprintf(stderr, "Could not dlopen(%s): %s\n", fn, dlerror());
    free(i);
    return;
  }

  i->name   = dlsym(i->handle, "_module_name");
  i->desc   = dlsym(i->handle, "_module_desc");
  type      = dlsym(i->handle, "_module_type");
  i->open   = dlsym(i->handle, "q_open");
  i->read   = dlsym(i->handle, "q_read");
  i->close  = dlsym(i->handle, "q_close");
  i->sysctl = dlsym(i->handle, "p_sysctl");
  
  if(type == NULL || strcmp(type, "DS") != 0 || i->open == NULL) {
    dlclose(i->handle);
    free(i);
    return;
  }

  printf("MM: \"%s\": %s (%s)\n", fn, i->name, i->desc);
  (**imtrav).Next = i;
  *imtrav = (**imtrav).Next;
  return;
}

//==[ End of file ]============================================================
